@@ -3,10 +3,14 @@ import uuid
 from copy import deepcopy
 from datetime import datetime
 from typing import Union, Optional, Dict, Literal, List, Type, Iterable
+
+import aiofiles as aiofiles
+
 from glQiwiApi.api import HttpXParser
 from glQiwiApi.configs import *
+from glQiwiApi.configs import ONLINE_COMMISSION_DATA, COMMISSION_TRANSFER
 from glQiwiApi.data import Response, InvalidCardNumber, WrapperData, Transaction, Identification, InvalidData, Limit, \
-    Bill
+    Bill, Commission
 
 
 class QiwiDataFormatter:
@@ -30,7 +34,7 @@ class QiwiDataFormatter:
             iterable_obj: Iterable,
             obj: Type,
             transfers: Dict[str, str]
-    ) -> Optional[List[Union[Transaction, Identification, Limit, Bill]]]:
+    ) -> Optional[List[Union[Transaction, Identification, Limit, Bill, Commission]]]:
         kwargs = {}
         objects = []
         for transaction in iterable_obj:
@@ -76,6 +80,12 @@ class QiwiWrapper:
                  phone_number: str,
                  secret_p2p: Optional[str] = None,
                  public_p2p: Optional[str] = None) -> None:
+        """
+        :param api_access_token: токен, полученный с https://qiwi.com/api
+        :param phone_number: номер вашего телефона с +
+        :param secret_p2p: секретный ключ, полученный с https://p2p.qiwi.com/
+        :param public_p2p: публичный ключ, полученный с https://p2p.qiwi.com/
+        """
         self._parser = HttpXParser()
         self.api_access_token = api_access_token
         self.phone_number = phone_number
@@ -93,7 +103,7 @@ class QiwiWrapper:
             self,
             trans_sum: Union[float, int],
             to_card: str
-    ) -> Response:
+    ) -> Union[Response, str]:
         """
         Метод для отправки средств на карту.
         Более подробная документация https://developer.qiwi.com/ru/qiwi-wallet-personal/#cards
@@ -113,7 +123,10 @@ class QiwiWrapper:
                 json=data.json,
                 get_json=True
         ):
-            return response
+            try:
+                return response.response_data.get('id')
+            except (KeyError, TypeError):
+                return response
 
     async def _detect_card_number(self, card_number: str) -> str:
         """
@@ -140,6 +153,31 @@ class QiwiWrapper:
             except KeyError:
                 raise InvalidCardNumber('Invalid card number or qiwi api is not response') from None
 
+    async def _detect_mobile_number(self, phone_number: str):
+        """
+        Метод для получения индентификатора телефона
+
+        https://developer.qiwi.com/ru/qiwi-wallet-personal/?python#cards
+        """
+        headers = deepcopy(DEFAULT_QIWI_HEADERS)
+        headers.update(
+            {
+                'Content-Type': 'application/x-www-form-urlencoded'
+            }
+        )
+        async for response in self._parser.fetch(
+                url='https://qiwi.com/mobile/detect.action',
+                headers=headers,
+                method='POST',
+                data={
+                    'phone': phone_number
+                }
+        ):
+            try:
+                return response.response_data.get('message')
+            except KeyError:
+                raise InvalidCardNumber('Invalid card number or qiwi api is not response') from None
+
     async def get_balance(self) -> Dict[str, int]:
         """Метод для получения баланса киви, принимает\n логин(номер телефона) и токен с https://qiwi.com/api"""
         headers = self._auth_token(deepcopy(DEFAULT_QIWI_HEADERS))
@@ -204,7 +242,7 @@ class QiwiWrapper:
             to_number: str,
             trans_sum: Union[str, float, int],
             currency: str = '643',
-            comment: str = '+comment+') -> Response:
+            comment: str = '+comment+') -> Union[str, Response]:
         """
         Метод для перевода денег на другой кошелек\n
         Подробная документация: https://developer.qiwi.com/ru/qiwi-wallet-personal/?python#p2p
@@ -229,7 +267,10 @@ class QiwiWrapper:
                 json=data.json,
                 headers=data.headers
         ):
-            return response
+            try:
+                return response.response_data['transaction']['id']
+            except (KeyError, TypeError):
+                return response
 
     async def transaction_info(
             self,
@@ -542,4 +583,63 @@ class QiwiWrapper:
                     obj=Bill
                 )[0]
             except IndexError:
-                raise ConnectionError('Не удалось создать p2p bill. Проверьте ваши токены.') from None
\ No newline at end of file
+                raise ConnectionError('Не удалось создать p2p bill. Проверьте ваши токены.') from None
+
+    async def get_receipt(
+            self,
+            transaction_id: Union[str, int],
+            transaction_type: Literal['IN', 'OUT', 'QIWI_CARD'],
+            file_path: Optional[str] = None
+    ) -> Union[bytearray, int]:
+        """
+        Метод для получения чека в формате байтов или файлом.
+
+        :param transaction_id: айди транзакции, можно получить при вызове методе to_wallet, to_card
+        :param transaction_type: тип транзакции может быть 'IN', 'OUT', 'QIWI_CARD'
+        :param file_path: путь к файлу, куда вы хотите сохранить чек, если не указан, возвращает байты
+        :return: pdf файл в байтовом виде или номер записанных байтов
+        """
+        headers = self._auth_token(deepcopy(DEFAULT_QIWI_HEADERS))
+        data = {
+            'type': transaction_type,
+            'format': 'PDF'
+        }
+        async for response in self._parser.fast().fetch(
+                url='https://edge.qiwi.com/payment-history/v1/transactions/' + transaction_id + '/cheque/file',
+                headers=headers,
+                method='GET',
+                data=data
+        ):
+            if not file_path:
+                return response.response_data
+            async with aiofiles.open(file_path + '.pdf', 'wb') as file:
+                return await file.write(response.response_data)
+
+    async def commission(self, to_account: str, pay_sum: Union[int, float]) -> Commission:
+        """
+        Возвращается полная комиссия QIWI Кошелька за платеж в пользу указанного провайдера
+        с учетом всех тарифов по заданному набору платежных реквизитов.
+
+        :param to_account: номер карты или киви кошелька
+        :param pay_sum: сумма, за которую вы хотите узнать коммиссию
+        :return: Commission object
+        """
+        headers = self._auth_token(deepcopy(DEFAULT_QIWI_HEADERS))
+        json_payload = deepcopy(ONLINE_COMMISSION_DATA)
+        json_payload['purchaseTotals']['total']['amount'] = pay_sum
+        json_payload['account'] = to_account.replace('+', '')
+        special_code = "99" if len(to_account.replace('+', '')) <= 15 else (
+            await self._detect_card_number(card_number=to_account))
+        async for response in self._parser.fast().fetch(
+                url='https://edge.qiwi.com/sinap/providers/' + special_code + '/onlineCommission',
+                headers=headers,
+                json=json_payload
+        ):
+            try:
+                return self._formatter.format_objects(
+                    iterable_obj=(response.response_data,),
+                    transfers=COMMISSION_TRANSFER,
+                    obj=Commission
+                )[0]
+            except IndexError:
+                raise ConnectionError('Не удалось получить коммиссию за платёж. Попробуйте ещё раз.') from None
